import { useState } from 'react';
import { SlatsAction, SlatsFixedValue, SlatsRanges } from '../components/shutterModels';

const slatsActions: { [key: string]: SlatsAction } = {
  DOWN: 'DOWN',
  STOP: 'STOP',
  UP: 'UP',
};

export const slatsRanges: { [key: string]: SlatsRanges } = {
  R1: 'R1',
  R2: 'R2',
  R3: 'R3',
  R4: 'R4',
};

const fixedSlatsValues: { [key: string]: SlatsFixedValue } = {
  V0: 0,
  V25: 25,
  V50: 50,
  V75: 75,
  V100: 100,
};

const rangeActionsManager = {
  [slatsActions.DOWN]: (value: number): SlatsRanges => {
    const isUnderOrEqual25 = value <= fixedSlatsValues.V25;
    const isAbove25 = value > fixedSlatsValues.V25;
    const isUnderOrEqual50 = value <= fixedSlatsValues.V50;
    const isAbove50 = value > fixedSlatsValues.V50;
    const isUnderOrEqual75 = value <= fixedSlatsValues.V75;

    const isBetween25And50 = isAbove25 && isUnderOrEqual50;
    const isBetween50And75 = isAbove50 && isUnderOrEqual75;

    return (
      (isUnderOrEqual25 && slatsRanges.R1) ||
      (isBetween25And50 && slatsRanges.R2) ||
      (isBetween50And75 && slatsRanges.R3) ||
      slatsRanges.R4
    );
  },
  [slatsActions.UP]: (value: number) => {
    const isUnder25 = value < fixedSlatsValues.V25;
    const isAbove25 = value >= fixedSlatsValues.V25;
    const isUnder50 = value < fixedSlatsValues.V50;
    const isAbove50 = value >= fixedSlatsValues.V50;
    const isUnder75 = value < fixedSlatsValues.V75;

    const isBetween25And50 = isAbove25 && isUnder50;
    const isBetween50And75 = isAbove50 && isUnder75;

    return (
      (isUnder25 && slatsRanges.R1) ||
      (isBetween25And50 && slatsRanges.R2) ||
      (isBetween50And75 && slatsRanges.R3) ||
      slatsRanges.R4
    );
  },
};

const getSlatsRange = (
  value: number,
  action: SlatsAction = slatsActions.DOWN,
): SlatsRanges => {
  return rangeActionsManager[action](value);
};

const slatsActionManagerR1 = {
  [slatsActions.DOWN]: fixedSlatsValues.V0,
  [slatsActions.UP]: fixedSlatsValues.V25,
};

const slatsActionManagerR2 = {
  [slatsActions.DOWN]: fixedSlatsValues.V25,
  [slatsActions.UP]: fixedSlatsValues.V50,
};

const slatsActionManagerR3 = {
  [slatsActions.DOWN]: fixedSlatsValues.V50,
  [slatsActions.UP]: fixedSlatsValues.V75,
};

const slatsActionManagerR4 = {
  [slatsActions.DOWN]: fixedSlatsValues.V75,
  [slatsActions.UP]: fixedSlatsValues.V100,
};

const slatsActionsState: {
  [x: string]: (action: SlatsAction) => SlatsFixedValue;
} = {
  [slatsRanges.R1]: (action: SlatsAction): SlatsFixedValue => {
    return slatsActionManagerR1[action];
  },
  [slatsRanges.R2]: (action: SlatsAction): SlatsFixedValue => {
    return slatsActionManagerR2[action];
  },
  [slatsRanges.R3]: (action: SlatsAction): SlatsFixedValue => {
    return slatsActionManagerR3[action];
  },
  [slatsRanges.R4]: (action: SlatsAction): SlatsFixedValue => {
    return slatsActionManagerR4[action];
  },
};

const getValueToSettingAction = (
  value: number,
  action: SlatsAction = slatsActions.DOWN,
): SlatsFixedValue => {
  const shutterRange = getSlatsRange(value, action);

  const getSlatsActionsState = slatsActionsState[shutterRange];
  return getSlatsActionsState(action);
};

const getValueToStopAction = (value: number) => value;

export const useSlats = () => {
  const [outputSlatsValue, setOutputSlatsValue] = useState<number>(0);

  const setSlatsValue = (value: number) => setOutputSlatsValue(value);

  const onSlatsAction = (
    value: number,
    action: SlatsAction,
  ): number | SlatsFixedValue => {
    const outputValueByAction: {
      [x: string]: (value: number, action?: SlatsAction) => number;
    } = {
      [slatsActions.STOP]: getValueToStopAction,
      [slatsActions.UP]: getValueToSettingAction,
      [slatsActions.DOWN]: getValueToSettingAction,
    };

    return outputValueByAction[action](value, action);
  };

  return {
    fixedSlatsValue: outputSlatsValue as SlatsFixedValue,
    setSlatsValue,
    onSlatsAction,
  };
};
